# -*- coding: utf-8 -*-


ONOFFOFF_MIN_POTENTIAL = 2
ONOFFOFF_MAX_POTENTIAL = 3
ONOFFOFF_MIN_NON = 3
ONOFFOFF_MAX_NON = 5

ONONOFF_MIN_POTENTIAL = 2
ONONOFF_MAX_POTENTIAL = 4
ONONOFF_MIN_NON = 3
ONONOFF_MAX_NON = 4

ATTR_CONFIG_SIZE = 24
ALL_CONFIG_SIZE = 48
